#!/usr/bin/env python3

from argparse import ArgumentParser
from pprint import pprint
from re import match

from vang.pio.shell import run_command
from vang.pio.synchronize_dirs import synchronize_dirs


def apply_patch(patch_repo, apply_repo, ref):
    print('Apply patch', ref, patch_repo, apply_repo, ref)
    rc, output = run_command(f'git checkout {ref}', True, patch_repo)
    print(output)

    synchronize_dirs(patch_repo, apply_repo)
    rc, output = run_command('git status', True, apply_repo)
    print(output)

    if 'nothing to commit' in output:
        rc, output = run_command(f'git tag -a {ref} -m {ref}', True, apply_repo)
        print(output)
    else:
        for cmd in [
            'git add --all',
            f'git commit -m {ref}',
            f'git tag -a {ref} -m {ref}',
        ]:
            rc, output = run_command(cmd, True, apply_repo)
            print(output)

    return ref


def get_refs(repo, ref_pattern):
    return [
        ref for ref in run_command('git tag', True, repo)[1].split('\n')
        if match(r'{}'.format(ref_pattern), ref)
    ]


def get_unpatched_refs(patchs_refs, applied_refs):
    return [p for p in patchs_refs if p not in applied_refs]


def is_valid(patchs_refs, applied_refs):
    return all([p == a for p, a in zip(patchs_refs, applied_refs)
                ]) and not len(applied_refs) > len(patchs_refs)


def main(patch_repo, ref_pattern, apply_repo):
    patchs_refs = get_refs(patch_repo, ref_pattern)
    applied_refs = get_refs(apply_repo, ref_pattern)
    if is_valid(patchs_refs, applied_refs):
        unpatched_refs = get_unpatched_refs(patchs_refs, applied_refs)
        applied_patches = []
        for ref in unpatched_refs:
            apply_patch(patch_repo, apply_repo, ref)
            applied_patches.append(ref)
        return applied_patches
    else:
        raise ValueError('Tags are not valid.')


def parse_args(args):
    parser = ArgumentParser(description='Create patches of refs and applies, ' +
                                        'commits and refs them in another repo.')
    parser.add_argument('ref_pattern', help='A ref pattern.')
    parser.add_argument('apply_repo', help='The repo to apply patches to.')
    parser.add_argument(
        '-p',
        '--patch_repo',
        help='The repo to patch from.',
        default='.',
    )
    parser.add_argument(
        '-o',
        '--output',
        help='A directory to put patches in.',
        default='./patch',
    )
    return parser.parse_args(args)


# rm -rf sign/patch; rm -rf lf-sign; md lf-sign; cd lf-sign; git init
# main(
#     '/Users/ei4577/slask/slask/PCS1806/sign',
#     'release.*',
#     '/Users/ei4577/slask/slask/PCS1806/lf-sign',
# )
# rm -rf lf-process.mortgage; md lf-process.mortgage; cd lf-process.mortgage; git init
the_applied_patches = main(
    '/Users/ei4577/slask/slask/PCS1806/process.mortgage',
    'release.*',
    '/Users/ei4577/slask/slask/PCS1806/lf-process.mortgage',
)
pprint(the_applied_patches)

# if __name__ == '__main__':  # pragma: no cover
#     args = parse_args(argv[1:])
#     main(args.patch_repo, args.ref_pattern, args.output, args.apply_repo)



#!/usr/bin/env python3
import argparse
from itertools import product
from multiprocessing.dummy import Pool
from sys import argv

from vang.bitbucket.api import call
from vang.bitbucket.utils import get_repo_specs


def set_repo_default_branch(spec, branch):
    request_data = {'id': f'refs/heads/{branch}'}
    return spec, call(
        f'/rest/api/1.0/projects/{spec[0]}/repos/{spec[1]}/branches/default',
        request_data,
        only_response_code=True,
        method='PUT',
    )


def set_default_branch(repo_specs, branch, max_processes=10):
    with Pool(processes=max_processes) as pool:
        return pool.starmap(set_repo_default_branch,
                            product(repo_specs, [branch]))


def main(branch, dirs=None, repos=None, projects=None):
    specs = get_repo_specs(dirs, repos, projects)
    for spec, response in set_default_branch(specs, branch):
        print(f'{spec[0]}/{spec[1]}: {response}')


def parse_args(args):
    parser = argparse.ArgumentParser(
        description='Set default branches for repositories in Bitbucket')
    parser.add_argument('branch', help='The branch to set')
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        '-d',
        '--dirs',
        nargs='*',
        default=['.'],
        help='Git directories to extract repo information from')
    group.add_argument(
        '-r', '--repos', nargs='*', help='Repos, e.g. key1/repo1 key2/repo2')
    group.add_argument(
        '-p', '--projects', nargs='*', help='Projects, e.g. key1 key2')

    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import argparse
from sys import argv

from vang.bitbucket.api import get_all
from vang.bitbucket.utils import get_repo_specs
from vang.core.core import pmap_unordered


def get_branches(repo_specs, branch='', max_processes=10):
    return pmap_unordered(
        lambda s: (s, get_all(f'/rest/api/1.0/projects/{s[0]}/repos/{s[1]}/branches',
                              {'filterText': branch})),
        repo_specs,
        processes=max_processes)


def main(branch='', name=False, dirs=None, repos=None, projects=None):
    specs = get_repo_specs(dirs, repos, projects)
    for spec, branches in get_branches(specs, branch):
        for b in branches:
            if name:
                print(b['displayId'])
            else:
                print(f'{spec[0]}/{spec[1]}: {b["displayId"]}')


def parse_args(args):
    parser = argparse.ArgumentParser(
        description='Get repository branches from Bitbucket')
    parser.add_argument('-b', '--branch', help='Branch filter', default='')
    parser.add_argument(
        '-n', '--name', help='Print only branch name', action='store_true')
    group = parser.add_mutually_exclusive_group()
    group.add_argument(
        '-d',
        '--dirs',
        nargs='*',
        default=['.'],
        help='Git directories to extract repo information from')
    group.add_argument(
        '-r', '--repos', nargs='*', help='Repos, e.g. key1/repo1 key2/repo2')
    group.add_argument(
        '-p', '--projects', nargs='*', help='Projects, e.g. key1 key2')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
import hashlib
import json
from argparse import ArgumentParser
from glob import glob
from sys import argv

from vang.artifactory import api
from vang.artifactory.utils import get_artifact_base_uri, get_pom_path
from vang.maven.pom import get_pom_info


def read_file(file_path):  # pragma: no cover
    with open(file_path, 'rb') as f:
        return f.read()


def get_checksums(the_bytes):
    return hashlib.md5(the_bytes).hexdigest(), hashlib.sha1(
        the_bytes).hexdigest(), hashlib.sha256(the_bytes).hexdigest()


def get_checksum_headers(md5, sha1, sha256):
    return {
        'X-Checksum-Md5': md5,
        'X-Checksum-Sha1': sha1,
        # sha256 Not yet supported...
        'X-Checksum-Sha256': sha256
    }


def get_pom_publish_name(pom_path, artifact_id, version):
    pom_name = pom_path.split('/')[-1]
    return pom_name if pom_name.split(
        '.')[-1] == 'pom' else f'{artifact_id}-{version}.pom'


def get_publish_data(artifact_base_uri, path, name):
    content = read_file(path)
    md5, sha1, sha256 = get_checksums(content)
    return {
        'content': content,
        'checksum_headers': get_checksum_headers(md5, sha1, sha256),
        'uri': f'{artifact_base_uri}/{name}'
    }


def publish_maven_artifact(repository, pom_dirs):
    for pom_dir in pom_dirs:
        pom_info = get_pom_info(get_pom_path(pom_dir))
        base_uri = get_artifact_base_uri(repository, pom_info['group_id'],
                                         pom_info['artifact_id'],
                                         pom_info['version'])

        publish_data = [get_publish_data(base_uri, pom_info['pom_path'],
                                         get_pom_publish_name(pom_info['pom_path'],
                                                              pom_info['artifact_id'],
                                                              pom_info['version']))] + \
                       [get_publish_data(base_uri, path, path.split('/')[-1])
                        for path in
                        glob(f'{pom_dir}/**/*.jar', recursive=True) +
                        glob(f'{pom_dir}/**/*.war', recursive=True)]

        yield [
            api.call(pd['uri'], pd['checksum_headers'], pd['content'], 'PUT')
            for pd in publish_data
        ]


def main(repository, dirs):
    for response in publish_maven_artifact(repository, dirs):
        print(json.dumps(json.loads(str(response).replace("'", '"')), indent=2))


def parse_args(args):
    parser = ArgumentParser(description='Publish maven artifact to Artifactory')
    parser.add_argument(
        'repository', help='Artifactory repository, e.g. z-release')
    parser.add_argument(
        '-d',
        '--dirs',
        nargs='*',
        default=['.'],
        help='Maven pom directories to extract artifact information from')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3
from argparse import ArgumentParser
from sys import argv

from vang.artifactory.api import call


def get_repo_content(repo_key):
    return call(f'/api/storage/{repo_key}?list&deep=1&listFolders=1')


def list_repo_content(repo_key, only_files=True):
    result = get_repo_content(repo_key)
    result = result['files']
    if only_files:
        result = [f['uri'] for f in result if not f['folder']]
    return result


def main(repo_key, only_files):
    for f in list_repo_content(repo_key, only_files):
        print(f)


def parse_args(args):
    parser = ArgumentParser(
        description='List repository content')
    parser.add_argument(
        'repo_key',
        help='Repository key, e.g. libs-release')
    parser.add_argument('-f', '--only_files', help='Print only ordinary files, i.e. not directories',
                        action='store_true')
    return parser.parse_args(args)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)



#!/usr/bin/env python3

import argparse
from sys import argv

from vang.tfs.api import call


def get_projects(organisations, project_specs=False, names=False):
    if not organisations:
        return []
    projects = [(o, p) for o in organisations
                for p in call(f'/{o}/_apis/projects?api-version=3.2')['value']]
    if names:
        return [project[1]['name'] for project in projects]
    if project_specs:
        return [f'{project[0]}/{project[1]["name"]}' for project in projects]

    return projects


def parse_args(args):
    parser = argparse.ArgumentParser(description='Get TFS projects')
    parser.add_argument(
        'organisations', nargs='+', help='TFS organisations, e.g organisation')
    group = parser.add_mutually_exclusive_group(required=False)
    group.add_argument(
        '-n', '--names', action='store_true', help='Get only project names')
    group.add_argument(
        '-p',
        '--project_specs',
        action='store_true',
        help='Get only organisation/project')

    return parser.parse_args(args)


def main(organisations,
         project_specs,
         names):
    for a_project in get_projects(
            organisations,
            project_specs,
            names):
        print(a_project)


if __name__ == '__main__':  # pragma: no cover
    main(**parse_args(argv[1:]).__dict__)
